PRODUTO_ARRECADACAO = 8

SEGMENTO_PREFEITURA = 1
SEGMENTO_SANEAMENTO = 2
SEGMENTO_ENERGIA_ELETRICA_GAS = 3
SEGMENTO_TELECOMUNICACOES = 4
SEGMENTO_ORGAOS_GOVERNAMENTAIS = 5
SEGMENTO_DEMAIS = 6
SEGMENTO_MULTAS_TRANSITO = 7
SEGMENTO_EXCLUSIVO_BANCO = 9

MODULO10_VALOR_EFETIVO = 6
MODULO10_QUANTIDADE_MOEDA = 7
MODULO11_VALOR_EFETIVO = 8
MODULO11_QUANTIDADE_MOEDA = 9

ARRECADACAO_GUICHE_COM_FATURA = '1'
"""1 – Guichê de Caixa com fatura/guia de arrecadação"""

ARRECADACAO_ELETRONICA_COM_FATURA = '2'
"""
2 – Arrecadação Eletrônica com fatura/guia de arrecadação (terminais de
auto - atendimento, ATM, home/office banking)
"""

ARRECADACAO_INTERNET_COM_FATURA = '3'
"""3 – Internet com fatura/guia de arrecadação"""

ARRECADACAO_OUTROS_COM_FATURA = '4'
"""4 – Outros meios com fatura/guia de arrecadação"""

ARRECADACAO_CORRESPONDENTES_COM_FATURA = '5'
"""5 – Correspondentes bancários com fatura/guia de arrecadação"""

ARRECADACAO_TELEFONE_COM_FATURA = '6'
"""6 – Telefone com fatura/guia de arrecadação"""

ARRECADACAO_LOTERICAS_COM_FATURA = '7'
"""7 – Casas lotéricas com fatura/guia de arrecadação"""

ARRECADACAO_GUICHE_SEM_FATURA = 'a'
"""a – Guichê de Caixa sem fatura/guia de arrecadação"""

ARRECADACAO_ELETRONICA_SEM_FATURA = 'b'
"""
b – Arrecadação Eletrônica sem fatura/guia de arrecadação (terminais de
auto - atendimento, ATM, home/office banking)
"""

ARRECADACAO_INTERNET_SEM_FATURA = 'c'
"""c – Internet sem fatura/guia de arrecadação"""

ARRECADACAO_CORRESPONDENTES_SEM_FATURA = 'd'
"""d – Correspondentes bancários sem fatura/guia de arrecadação"""

ARRECADACAO_TELEFONE_SEM_FATURA = 'e'
"""e – Telefone sem fatura/guia de arrecadação"""

ARRECADACAO_OUTROS_SEM_FATURA = 'f'
"""f – Outros meios sem fatura/guia de arrecadação"""

ARRECADACAO_LOTERICAS_SEM_FATURA = 'g'
"""g – Casas lotéricas sem fatura/guia de arrecadação"""

PAGAMENTO_DINHEIRO = '1'
"""1 – Dinheiro"""

PAGAMENTO_CHEQUE = '2'
"""2 – Cheque"""

PAGAMENTO_NAO_IDENTIFICADO = '3'
"""3 – Não identificado/outras formas"""

PAGAMENTO_OUTRAS_FORMAS = '3'
"""3 – Não identificado/outras formas"""
